﻿using System.Security.Claims;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Authentication.Cookies;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Website.Data;
using Website.Models.Auth;
using Website.ViewModels;

namespace Website.Controllers {
	public class AccountController:Controller {
		private readonly IAuthenticationProvider _authenticationProvider;

		public AccountController(IAuthenticationProvider authenticationProvider) {
			_authenticationProvider = authenticationProvider;
		}

		[Authorize]
		public IActionResult Index() {
			var model = new AccountViewModel();
			return View(model);
		}

		[HttpGet]
		public IActionResult Login(string returnUrl, bool failedAttempt = false) {
			var model = new LoginViewModel {
				ReturnUrl = returnUrl,
				FailedAttempt = failedAttempt
			};
			return View(model);
		}

		[HttpPost]
		public async Task<IActionResult> Login(LoginRequest request) {
			try {
				var user = await _authenticationProvider.Authenticate(request);
				return user != null
					? await SetIdentityAndRedirect(request.ReturnUrl, user)
					: Login(request.ReturnUrl, true);
			}
			catch {
				return Login(request.ReturnUrl, true);
			}
		}

		private async Task<IActionResult> SetIdentityAndRedirect(string returnUrl, User user) {
			var identity = new ClaimsIdentity(CookieAuthenticationDefaults.AuthenticationScheme);
			identity.AddClaim(new Claim(ClaimTypes.Name, user.Username));
			identity.AddClaim(new Claim(ClaimTypes.Email, user.Username));

			var principal = new ClaimsPrincipal(identity);
			await HttpContext.SignInAsync(CookieAuthenticationDefaults.AuthenticationScheme, principal);

			return string.IsNullOrEmpty(returnUrl)
				? (IActionResult)RedirectToAction(nameof(Index))
				: Redirect(returnUrl);
		}

		public async Task<IActionResult> Logout() {
			await HttpContext.SignOutAsync();
			return RedirectToAction(nameof(Login));
		}

		[HttpPost(nameof(UpdatePassword))]
		public async Task<IActionResult> UpdatePassword(UpdatePasswordRequest request) {
			request.Username = HttpContext.User.FindFirst(ClaimTypes.Name).Value;
			TempData["updatePassword"] = request.NewPassword == request.ConfirmPassword && await _authenticationProvider.UpdateUserPassword(request);
			return RedirectToAction(nameof(Index));
		}
	}
}