﻿using System;
using System.IO;
using System.Linq;
using JavaScriptEngineSwitcher.Extensions.MsDependencyInjection;
using JavaScriptEngineSwitcher.V8;
using Microsoft.AspNetCore.Authentication.Cookies;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Robware.Lib.ApiClient;
using Website.Data;

namespace Website
{
	public class Startup
	{
		public Startup(IConfiguration configuration)
		{
			Configuration = configuration;
		}

		public IConfiguration Configuration { get; }

		public void ConfigureServices(IServiceCollection services)
		{
			services.Configure<CookiePolicyOptions>(options =>
			{
				options.CheckConsentNeeded = context => true;
				options.MinimumSameSitePolicy = SameSiteMode.None;
			});

			services.AddSingleton(Configuration);

			services.AddSingleton(Configuration.GetSection("cacheDurations").Get<CacheDurations>());

			void AddHttpClientWithApiKey<TInterface, TImplementation>(string uriConfigKey, string extraUriString = "") where TInterface : class where TImplementation : class, TInterface =>
				services.AddHttpClient<TInterface, TImplementation>(client =>
				{
					client.BaseAddress = new Uri(Configuration[uriConfigKey] + extraUriString);
					client.DefaultRequestHeaders.Add("x-api-key", new[] { Configuration["authApiKey"] });
				});
			AddHttpClientWithApiKey<IGitApi, GitApi>("gitApiEndpoint");
			AddHttpClientWithApiKey<IBlogApi, BlogApi>("blogApiEndpoint");
			AddHttpClientWithApiKey<IAuthenticationProvider, AuthenticationProvider>("authApiEndpoint");
			AddHttpClientWithApiKey<IApiKeyManager, ApiKeyManager>("authApiEndpoint", "api/");
			AddHttpClientWithApiKey<IMailboxesApi, MailboxesApi>("mailboxesApiEndpoint");

			services.AddAuthentication(CookieAuthenticationDefaults.AuthenticationScheme).AddCookie();

			services.AddMvc(options => options.EnableEndpointRouting = false)
#if DEBUG
			        .AddRazorRuntimeCompilation()
#endif
				;

			services.AddJsEngineSwitcher(options => options.DefaultEngineName = V8JsEngine.EngineName).AddV8();

			static string[] FindAllScss(string path) {
				var scssFiles = Directory.GetFiles(path, "*.scss");
				var folders = Directory.GetDirectories(path);
				foreach (var folder in folders)
					scssFiles = scssFiles.Union(FindAllScss(folder)).ToArray();
				return scssFiles;
			}

			services.AddWebOptimizer(pipeline => {
				pipeline.CompileScssFiles();
			});
		}

		public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
		{
			if (env.IsDevelopment())
			{
				app.UseDeveloperExceptionPage();
			}
			else
			{
				app.UseExceptionHandler("/Error/ServerError");
				// The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
				app.UseHsts();
			}

			app.UseStatusCodePagesWithReExecute("/Error/PageNotFound")
			   .UseHttpsRedirection()
			   .UseWebOptimizer()
			   .UseStaticFiles()
			   .UseAuthentication()
			   .UseMvc(routes => {
				   routes.MapRoute(
				                   name: "blogPages",
				                   template: "blog/{action}/{page:int}",
				                   defaults: new {controller = "Blog", action = "Page", page = 1});
				   routes.MapRoute(
				                   name: "blogView",
				                   template: "blog/view/{url}",
				                   defaults: new {controller = "Blog", action = "Entry"});
				   routes.MapRoute(
				                   name: "default",
				                   template: "{controller=Home}/{action=Index}/{id?}");
			   });
		}
	}
}
